@@ -140,6 +140,9 @@ struct libinput {
 	struct list device_group_list;
 
 	uint64_t last_event_time;
+
+	bool quirks_initialized;
+	struct quirks_context *quirks;
 };
 
 typedef void (*libinput_seat_destroy_func) (struct libinput_seat *seat);
@@ -427,6 +430,9 @@ libinput_init(struct libinput *libinput,
 	      const struct libinput_interface_backend *interface_backend,
 	      void *user_data);
 
+void
+libinput_init_quirks(struct libinput *libinput);
+
 struct libinput_source *
 libinput_add_fd(struct libinput *libinput,
 		int fd,
@@ -542,6 +548,12 @@ touch_notify_touch_up(struct libinput_device *device,
 		      int32_t slot,
 		      int32_t seat_slot);
 
+void
+touch_notify_touch_cancel(struct libinput_device *device,
+			  uint64_t time,
+			  int32_t slot,
+			  int32_t seat_slot);
+
 void
 touch_notify_frame(struct libinput_device *device,
 		   uint64_t time);
@@ -650,7 +662,7 @@ libinput_now(struct libinput *libinput)
 }
 
 static inline struct device_float_coords
-device_delta(struct device_coords a, struct device_coords b)
+device_delta(const struct device_coords a, const struct device_coords b)
 {
 	struct device_float_coords delta;
 
@@ -661,7 +673,7 @@ device_delta(struct device_coords a, struct device_coords b)
 }
 
 static inline struct device_float_coords
-device_average(struct device_coords a, struct device_coords b)
+device_average(const struct device_coords a, const struct device_coords b)
 {
 	struct device_float_coords average;
 
@@ -672,7 +684,7 @@ device_average(struct device_coords a, struct device_coords b)
 }
 
 static inline struct device_float_coords
-device_float_delta(struct device_float_coords a, struct device_float_coords b)
+device_float_delta(const struct device_float_coords a, const struct device_float_coords b)
 {
 	struct device_float_coords delta;
 
@@ -683,7 +695,7 @@ device_float_delta(struct device_float_coords a, struct device_float_coords b)
 }
 
 static inline struct device_float_coords
-device_float_average(struct device_float_coords a, struct device_float_coords b)
+device_float_average(const struct device_float_coords a, const struct device_float_coords b)
 {
 	struct device_float_coords average;
 
@@ -694,25 +706,25 @@ device_float_average(struct device_float_coords a, struct device_float_coords b)
 }
 
 static inline bool
-device_float_is_zero(struct device_float_coords coords)
+device_float_is_zero(const struct device_float_coords coords)
 {
 	return coords.x == 0.0 && coords.y == 0.0;
 }
 
 static inline double
-normalized_length(struct normalized_coords norm)
+normalized_length(const struct normalized_coords norm)
 {
 	return hypot(norm.x, norm.y);
 }
 
 static inline bool
-normalized_is_zero(struct normalized_coords norm)
+normalized_is_zero(const struct normalized_coords norm)
 {
 	return norm.x == 0.0 && norm.y == 0.0;
 }
 
 static inline double
-length_in_mm(struct phys_coords mm)
+length_in_mm(const struct phys_coords mm)
 {
 	return hypot(mm.x, mm.y);
 }
@@ -775,7 +787,7 @@ xy_get_direction(double x, double y)
 }
 
 static inline uint32_t
-phys_get_direction(struct phys_coords mm)
+phys_get_direction(const struct phys_coords mm)
 {
 	return xy_get_direction(mm.x, mm.y);
 }
@@ -785,7 +797,7 @@ phys_get_direction(struct phys_coords mm)
  * assumption: coordinates are normalized to one axis resolution.
  */
 static inline uint32_t
-device_float_get_direction(struct device_float_coords coords)
+device_float_get_direction(const struct device_float_coords coords)
 {
 	return xy_get_direction(coords.x, coords.y);
 }
