@@ -382,6 +382,13 @@ libinput_udev_assign_seat(struct libinput *libinput,
 {
 	struct udev_input *input = (struct udev_input*)libinput;
 
+	/* We cannot do this during udev_create_context because the log
+	 * handler isn't set up there but we really want to log to the right
+	 * place if the quirks run into parser errors. So we have to do it
+	 * here since we can expect the log handler to be set up by now.
+	 */
+	libinput_init_quirks(libinput);
+
 	if (!seat_id)
 		return -1;
 
